@@ -14,9 +14,9 @@ import mouse
 from collections import namedtuple
 import pyperclip
 import codecs
-from .core import *
-
-__all__ = ['Recorder', 'overlay_add_mode_icon', 'overlay_add_progress_icon']
+from .core import path_separator, type_separator, Strategy, is_int, \
+                    get_wrapper_path, get_entry_list, get_entry, match_entry_list, get_sorted_region, \
+                    find_element, read_config_file
 
 ElementEvent = namedtuple('ElementEvent', ['strategy', 'rectangle', 'path'])
 SendKeysEvent = namedtuple('SendKeysEvent', ['line'])
@@ -60,9 +60,9 @@ def compute_dx_dy(x, y, rectangle):
 	return (dx, dy)
 
 
-def write_in_file(events, relative_coordinate_mode=False, menu_click=False):
+def write_in_file(events, relative_coordinate_mode=False):
 	from pathlib import Path
-	home_dir = Path.home() / Path('Pywinauto recorder')
+	home_dir = Path.home() / 'Pywinauto recorder'
 	home_dir.mkdir(parents=True, exist_ok=True)
 	record_file_name = home_dir / Path('recorded ' + time.asctime().replace(':', '_') + '.py')
 	print('Recording in file: ' + str(record_file_name.absolute()))
@@ -83,11 +83,11 @@ def write_in_file(events, relative_coordinate_mode=False, menu_click=False):
 					e_i_window = entry_list[0]
 					if e_i_window != common_window:
 						common_window = e_i_window
-						script += '\nwith Window(u"' + escape_special_char(common_window) + '"):\n'
+						script += '\nwith UIPath(u"' + escape_special_char(common_window) + '"):\n'
 					e_i_region = path_separator.join(entry_list[1:])
 					if e_i_region != common_region and e_i_region:
 						common_region = e_i_region
-						script += '\twith Region(u"' + escape_special_char(common_region) + '"):\n'
+						script += '\twith UIPath(u"' + escape_special_char(common_region) + '"):\n'
 					else:
 						common_region = ''
 		if type(e_i) in [SendKeysEvent, MouseWheelEvent, DragAndDropEvent, ClickEvent, FindEvent, MenuEvent]:
@@ -571,7 +571,7 @@ class Recorder(Thread):
 		self.event_list = []
 		self._copy_count = 0
 		self._mode = "Info"
-		self._process_menu_click_mode = True
+		self._process_menu_click_mode = False
 		self._smart_mode = False
 		self._relative_coordinate_mode = False
 		self.wrapper_old_info_tip = None
@@ -610,7 +610,7 @@ class Recorder(Thread):
 						previous_wrapper_path2 = wrapper_path2
 						
 						entry_list2 = get_entry_list(wrapper_path2)
-						unique_candidate2, _ = find_element(self.desktop, entry_list2, window_candidates=[])
+						unique_candidate2, _ = find_element(entry_list2)
 						
 						if unique_candidate2 is not None:
 							r = wrapper2_rectangle
@@ -655,52 +655,59 @@ class Recorder(Thread):
 				if type(self.event_list[-1]) == mouse.MoveEvent:
 					self.event_list = self.event_list[:-1]
 			self.event_list.append(mouse_event)
-
+			
+	def __start_stop_recording_by_key(self):
+		if self.mode != "Record":
+			self.started_recording_with_keyboard = True
+			self.start_recording()
+		else:
+			self.stop_recording()
+	
+	def __start_stop_displaying_info_by_key(self):
+		if self.mode == "Info":
+			self.mode = "Stop"
+		else:
+			self.mode = "Info"
+			
+	def __display_found_elemenet_by_key(self):
+		if self.last_element_event:
+			self._copy_count = 2
+			x, y = win32api.GetCursorPos()
+			l_e_e = self.last_element_event
+			dx, dy = compute_dx_dy(x, y, l_e_e.rectangle)
+			str_dx, str_dy = "{:.2f}".format(round(dx * 100, 2)), "{:.2f}".format(round(dy * 100, 2))
+			i = l_e_e.path.find(path_separator)
+			window_title = l_e_e.path[0:i]
+			# element_path = l_e_e.path[i+len(path_separator):]
+			p = get_relative_path(window_title, l_e_e.path)
+			code = 'with UIPath(u"' + escape_special_char(window_title) + '"):\n'
+			code += '\twrapper = find(u"' + escape_special_char(p)
+			if self.relative_coordinate_mode and eval(str_dx) != 0 and eval(str_dy) != 0:
+				code += '%(' + str_dx + ',' + str_dy + ')'
+			code += '")\n'
+			code += '\twrapper.draw_outline()\n'
+			pyperclip.copy(code)
+			if self.event_list:
+				self.event_list.append(FindEvent(path=l_e_e.path, dx=dx, dy=dy, time=time.time()))
+				
 	def __key_on(self, e):
+		key_to_scan_codes = keyboard.key_to_scan_codes
 		if (
-				e.name == 'r' and e.event_type == 'up'
-				and keyboard.key_to_scan_codes("alt")[0] in keyboard._pressed_events
-				and keyboard.key_to_scan_codes("ctrl")[0] in keyboard._pressed_events):
-			if self.mode != "Record":
-				self.started_recording_with_keyboard = True
-				self.start_recording()
-			else:
-				self.stop_recording()
+				(e.name, e.event_type) == ('r', 'up') and
+				set([key_to_scan_codes("alt")[0], key_to_scan_codes("ctrl")[0]]).issubset(keyboard._pressed_events)):
+			self.__start_stop_recording_by_key()
 		elif (
-				(e.name == 's') and (e.event_type == 'up')
-				and keyboard.key_to_scan_codes("alt")[0] in keyboard._pressed_events
-				and keyboard.key_to_scan_codes("ctrl")[0] in keyboard._pressed_events):
+				(e.name, e.event_type) == ('s', 'up') and
+				set([key_to_scan_codes("alt")[0], key_to_scan_codes("ctrl")[0]]).issubset(keyboard._pressed_events)):
 			self.smart_mode = not self.smart_mode
 		elif (
-				(e.name == 'F') and (e.event_type == 'up')
-				and keyboard.key_to_scan_codes("shift")[0] in keyboard._pressed_events
-				and keyboard.key_to_scan_codes("ctrl")[0] in keyboard._pressed_events):
-			if self.last_element_event:
-				self._copy_count = 2
-				x, y = win32api.GetCursorPos()
-				l_e_e = self.last_element_event
-				dx, dy = compute_dx_dy(x, y, l_e_e.rectangle)
-				str_dx, str_dy = "{:.2f}".format(round(dx * 100, 2)), "{:.2f}".format(round(dy * 100, 2))
-				i = l_e_e.path.find(path_separator)
-				window_title = l_e_e.path[0:i]
-				# element_path = l_e_e.path[i+len(path_separator):]
-				p = get_relative_path(window_title, l_e_e.path)
-				code = 'with Window(u"' + escape_special_char(window_title) + '"):\n'
-				code += '\twrapper = find(u"' + escape_special_char(p)
-				if self.relative_coordinate_mode and eval(str_dx) != 0 and eval(str_dy) != 0:
-					code += '%(' + str_dx + ',' + str_dy + ')'
-				code += '")\n'
-				pyperclip.copy(code)
-				if self.event_list:
-					self.event_list.append(FindEvent(path=l_e_e.path, dx=dx, dy=dy, time=time.time()))
+				(e.name, e.event_type) == ('F', 'up') and
+				set([key_to_scan_codes("shift")[0], key_to_scan_codes("ctrl")[0]]).issubset(keyboard._pressed_events)):
+			self.__display_found_elemenet_by_key()
 		elif (
-				(e.name == 'D') and (e.event_type == 'up')
-				and keyboard.key_to_scan_codes("shift")[0] in keyboard._pressed_events
-				and keyboard.key_to_scan_codes("ctrl")[0] in keyboard._pressed_events):
-			if self.mode == "Info":
-				self.mode = "Stop"
-			else:
-				self.mode = "Info"
+				(e.name, e.event_type) == ('D', 'up') and
+				set([key_to_scan_codes("shift")[0], key_to_scan_codes("ctrl")[0]]).issubset(keyboard._pressed_events)):
+			self.__start_stop_displaying_info_by_key()
 		elif self.mode == "Record":
 			self.event_list.append(e)
 
@@ -763,7 +770,8 @@ class Recorder(Thread):
 			geometry=oaam.Shape.rectangle, thickness=0
 		)
 		self.info_overlay.add(
-			geometry=oaam.Shape.rectangle, x=info_left, y=info_top - 2 + tooltip_height, width=tooltip_width, height=tooltip2_height,
+			geometry=oaam.Shape.rectangle, x=info_left, y=info_top - 2 + tooltip_height,
+			width=tooltip_width, height=tooltip2_height,
 			thickness=1, color=(0, 0, 0), brush=oaam.Brush.solid, brush_color=(180, 254, 255))
 		self.info_overlay.add(
 			x=info_left + 5, y=info_top + tooltip_height, width=tooltip_width-7,
@@ -847,6 +855,48 @@ class Recorder(Thread):
 			loop_duration = time.time() - self._loop_t0
 		self.info_overlay.refresh()
 
+	# Ne fonctionne pas dans tous les cas car un rectangle pere n'englobe pas forcement un rectangle fils
+	# (par exemple un TreeItem) Mais peut être utilisé en utime recour
+	'''
+	def my_from_point(self, x, y, wrapper=None):
+		def get_children_of_children_whith_empty_rectangle(wrapper_children):
+			children_of_children_whith_empty_rectangle = []
+			for child in wrapper_children:
+				child_rectangle = child.rectangle()
+				if child_rectangle.width() ==0 or child_rectangle.width()==0:
+					children_of_children_whith_empty_rectangle += child.children()
+			return children_of_children_whith_empty_rectangle
+		def get_children_of_children_not_in_parent_rectangle(wrapper_children):
+			children_of_children_whith_empty_rectangle = []
+			for child in wrapper_children:
+				child_rectangle = child.rectangle()
+				grandchildren = child.children()
+				if grandchildren:
+					x = grandchildren[0].rectangle().left
+					y = grandchildren[0].rectangle().top
+					if not ((child_rectangle.left < x < child_rectangle.right) and (child_rectangle.top < y < child_rectangle.bottom)):
+						return grandchildren
+			return []
+		if not wrapper:
+			wrapper = self.desktop.top_from_point(x, y)
+		wrapper_children = wrapper.children()
+		for child in wrapper_children:
+			child_rectangle = child.rectangle()
+			if (child_rectangle.left < x < child_rectangle.right) and (child_rectangle.top < y < child_rectangle.bottom):
+				return self.my_from_point(x, y, wrapper=child)
+		for grandchild in get_children_of_children_whith_empty_rectangle(wrapper_children):
+			child_rectangle = grandchild.rectangle()
+			if (child_rectangle.left < x < child_rectangle.right) and (child_rectangle.top < y < child_rectangle.bottom):
+				return self.my_from_point(x, y, wrapper=grandchild)
+			
+		for grandchild in get_children_of_children_not_in_parent_rectangle(wrapper_children):
+			child_rectangle = grandchild.rectangle()
+			if (child_rectangle.left < x < child_rectangle.right) and (child_rectangle.top < y < child_rectangle.bottom):
+				return self.my_from_point(x, y, wrapper=grandchild)
+
+		return wrapper
+	'''
+	
 	def run(self):
 		import comtypes.client
 		print("COMPTYPES CACHE FOLDER:", comtypes.client._code_cache._find_gen_dir())
@@ -871,6 +921,7 @@ class Recorder(Thread):
 		unique_wrapper_path = None
 		strategies = [Strategy.unique_path, Strategy.array_2D, Strategy.array_1D]
 		i_strategy = 0
+		#strategy_unique_path_again_done = False
 		while self.mode != "Quit":
 			i = i + 1
 			try:
@@ -878,6 +929,7 @@ class Recorder(Thread):
 				self.main_overlay.clear_all()
 				cursor_pos = win32api.GetCursorPos()
 				wrapper = self.desktop.from_point(*cursor_pos)
+				#wrapper = self.my_from_point(*cursor_pos)
 				if wrapper is None:
 					continue
 				wrapper_path = get_wrapper_path(wrapper)
@@ -891,16 +943,59 @@ class Recorder(Thread):
 						if i_strategy >= len(strategies):
 							i_strategy = len(strategies) - 1
 				else:
+					# strategy_unique_path_again_done = False
 					i_strategy = 0
 					previous_wrapper_path = wrapper_path
 					entry_list = get_entry_list(wrapper_path)
-					unique_candidate, elements = find_element(self.desktop, entry_list, window_candidates=[])
+					unique_candidate, elements = find_element(entry_list)
+				#if wrapper_path == previous_wrapper_path and unique_wrapper_path:
+				#	strategy = Strategy.unique_path_again
+				# else:
+				#	strategy = strategies[i_strategy]
 				strategy = strategies[i_strategy]
 				unique_wrapper_path = None
 				# *** ----> this block of code must start a new while iteration if mouse cursor is outside wrapper rectangle
 				# => add tests to leave if mouse cursor is outside wrapper rectangle
 				wrapper_rectangle = wrapper.rectangle()
-				if strategy == Strategy.unique_path:
+				
+				'''
+				# TODO: cette strategie n'est utile que pour palier à self.desktop.from_point(*cursor_pos) qui ne fn pas
+				# dans tous les cas (par exemple CMD.exe). L'autre solution est de reimplementer from_point
+				if strategy == Strategy.unique_path_again and unique_candidate and not strategy_unique_path_again_done:
+					x_new, y_new = win32api.GetCursorPos()
+					if not ((wrapper_rectangle.left < x_new < wrapper_rectangle.right) and (
+							wrapper_rectangle.top < y_new < wrapper_rectangle.bottom)):
+						i_strategy = 0
+						continue
+					better_candidates = []
+					better_father_candidates = []
+					curent_wrapper = unique_candidate.top_level_parent()
+					for curent_wrapper_child in curent_wrapper.children():
+						rect_child = curent_wrapper_child.rectangle()
+						if ((rect_child.left < x_new < rect_child.right) and (rect_child.top < y_new < rect_child.bottom)):
+							better_father_candidates.append(curent_wrapper_child)
+							
+					while better_father_candidates:
+						curent_wrapper = better_father_candidates.pop()
+						curent_wrapper_children = curent_wrapper.children()
+						if curent_wrapper_children:
+							for curent_wrapper_child in curent_wrapper_children:
+								rect_child = curent_wrapper_child.rectangle()
+								if ((rect_child.left < x_new < rect_child.right) and (
+										rect_child.top < y_new < rect_child.bottom)):
+									better_father_candidates.append(curent_wrapper_child)
+						else:
+							better_candidates.append(curent_wrapper)
+					if better_candidates:
+						better_candidates.sort(key=lambda widget: widget.rectangle().width() * widget.rectangle().height())
+						unique_candidate = better_candidates[0]
+						wrapper = unique_candidate
+						wrapper_rectangle = wrapper.rectangle()
+					strategy_unique_path_again_done = True
+				'''
+				
+				#if strategy in [Strategy.unique_path, Strategy.unique_path_again]:
+				if strategy in [Strategy.unique_path]:
 					x_new, y_new = win32api.GetCursorPos()
 					if not ((wrapper_rectangle.left < x_new < wrapper_rectangle.right) and (
 							wrapper_rectangle.top < y_new < wrapper_rectangle.bottom)):
@@ -916,8 +1011,10 @@ class Recorder(Thread):
 						for e in elements:
 							r = e.rectangle()
 							self.main_overlay.add(
-								geometry=oaam.Shape.rectangle, x=r.left, y=r.top, width=r.width(), height=r.height(),
+								geometry=oaam.Shape.rectangle, x=r.left, y=r.top, width=r.width(),
+								height=r.height(),
 								thickness=1, color=(0, 128, 0), brush=oaam.Brush.solid, brush_color=(255, 0, 0))
+
 				if strategy == Strategy.array_1D and elements:
 					x_new, y_new = win32api.GetCursorPos()
 					if not ((wrapper_rectangle.left < x_new < wrapper_rectangle.right) and (
@@ -979,10 +1076,12 @@ class Recorder(Thread):
 					self._copy_count = self._copy_count - 1
 				
 				self.main_overlay.refresh()
-				if self.mode == "Info":
+				if self.mode == "Info" and (unique_candidate or elements) :
 					self.__display_info_tip(*cursor_pos, wrapper)
 				else:
-					time.sleep(0.005)  # main_overlay.clear_all() doit attendre la fin de main_overlay.refresh()
+					self.info_overlay.clear_all()
+					self.info_overlay.refresh()
+					time.sleep(0.1)  # main_overlay.clear_all() doit attendre la fin de main_overlay.refresh()
 			except Exception as e:
 				exc_type, exc_value, exc_traceback = sys.exc_info()
 				print(repr(traceback.format_exception(exc_type, exc_value, exc_traceback)))
@@ -1068,3 +1167,5 @@ class Recorder(Thread):
 		self.mode = 'Quit'
 		self.join()
 		print("Quit")
+
+read_config_file()
\ No newline at end of file
